
----------------------------------------------
-- Variables
----------------------------------------------

g_BBar = {};					-- Main storage object

-- Static Variables
g_BBar.BASEBARSZ = 80;
g_BBar.BASEBARMINSZ = 58;
g_BBar.TOTALITEMS = 48;			-- Total number of items including titles (size of BBarObjects_Shown array)
g_BBar.MINCURRENCYID = 10;		-- Smallest possible currency ID (below this are title IDs)
g_BBar.MONEYID = 11;			-- ID of the money object
g_BBar.MONEYPOS = 2;			-- Position in BBarObjects of the money object
g_BBar.BARRESIZESTEP = 20;		-- Higher number = faster movement
g_BBar.BARREPSHIFT = 28;		-- Distance the rep bars are shifted to the left
g_BBar.MAXPROFESSIONS = 6;		-- Number of professions

-- Currency and custom items
g_BBar.BBarObjects = {1, 11, 2, 395, 396, 392, 390, 3, 61, 361, 698, 402, 81, 4, 515, 241, 416, 391, 614, 615, 738, 697, 752, 789, 777, 776, 823, 944, 980, 824, 994, 5, 384, 398, 394, 385, 397, 393, 399, 400, 401, 676, 677, 754, 821, 828, 829, 6};
g_BBar.BBarObjects_Shown = {};
g_BBar.BBarObjectsItems = {};
g_BBar.BBarObjectsItems_Shown = {};
g_BBar.BBarObjectsReps_Faction = {};	-- List of displayed faction IDs
g_BBar.BBarObjectsProfs_Shown = {0, 0, 0, 0, 0, 0};

-- Other
g_BBar.Setup = 0;
g_BBar.OnetimeEventUpdate = 0;
g_BBar.MoneyFrameObject = nil;
g_BBar.BBarTitleRegion = nil;
g_BBar.InfoScrollFrame = nil;
g_BBar.CurrencyScrollFrame = nil;
g_BBar.RepScrollFrame = nil;
g_BBar.ProfFrame = nil;
g_BBar.OptionsFrame = nil;

g_BBar.InfoBaseHeight = 0;
g_BBar.InfoCaptureTimeChat = 0;
g_BBar.InfoCaptureTimeWrite = 0;
g_BBar.InfoShowExpired = 0;
g_BBar.CurrentSettingsTab = 0;
g_BBar.LastSettingsItem = nil;
g_BBar.DefaultSettingsheight = 0;
g_BBar.UpdateBarPosition = 0;		-- Skip updating it the first time or it'll be positioned wrong
g_BBar.UpdateTooltip = 0;			-- GameTooltip mouse link
g_BBar.PosStorage = {};

g_BBar.CurentGold = 0;
g_BBar.CurrentDay = 0;
g_BBar.GoldSession = {0, 0};		-- Gain, loss
g_BBar.GoldToday = {0, 0};			-- Gain, loss
g_BBar.GoldYesterday = {0, 0};		-- Gain, loss
g_BBar.GoldThisWeek = {0, 0};		-- Gain, loss
g_BBar.GoldLastWeek = {0, 0};		-- Gain, loss

-- Resize animation
g_BBar.rsCurrentSize = 1;		-- 0 = Small, 1 = Large
g_BBar.rsAnimating = 0;			-- 1 = Shrinking or Growing
g_BBar.rsAnimatingSize = 0;		-- Current width
g_BBar.rsStoredWidth = 0;


----------------------------------------------
-- Functions
----------------------------------------------


SLASH_BBAR_RESET1 = '/bbar';
SlashCmdList["BBAR_RESET"] = function()
	_G["BBarFrame"]:ClearAllPoints();
	_G["BBarFrame"]:SetPoint("CENTER", 0, 0);
	_G["BBarSettingsFrame"]:ClearAllPoints();
	_G["BBarSettingsFrame"]:SetPoint("CENTER", 0, 0);
end

local oldChatFrame_DisplayTimePlayed;

function BBar_OnLoad(self)

	g_BBar.BBarTitleRegion = self:CreateTitleRegion();

	-- Store the money frame object and remove the scripts
	g_BBar.MoneyFrameObject = _G[self:GetName().."MoneyFrame"];
	_G[g_BBar.MoneyFrameObject:GetName().."CopperButton"]:SetScript("OnClick", nil);
	_G[g_BBar.MoneyFrameObject:GetName().."SilverButton"]:SetScript("OnClick", nil);
	_G[g_BBar.MoneyFrameObject:GetName().."GoldButton"]:SetScript("OnClick", nil);

	-- Create a button over the money frame to manage the tooltip
	local tooltip = CreateFrame("Button", g_BBar.MoneyFrameObject:GetName().."TT", g_BBar.MoneyFrameObject);
	tooltip:SetFrameLevel(self:GetFrameLevel() + 4);
	tooltip:SetAllPoints();
	tooltip:SetScript("OnEnter", function() BBar_ShowGoldTooltip(tooltip);	end);
	tooltip:SetScript("OnLeave", function() BBarMoneyToolTip:Hide();		end);

--	_G["BBarSettingsFrameScrollBack"]:SetFrameLevel(1);

	-- Hook the container item clicking for right click adding.
	hooksecurefunc("HandleModifiedItemClick", BBar_HandleModifiedItemClick);

	-- Hook the chat window time display to prevent writing when i call for the time
	oldChatFrame_DisplayTimePlayed = ChatFrame_DisplayTimePlayed;
	ChatFrame_DisplayTimePlayed = BBar_ChatFrame_DisplayTimePlayed;
	
	ValuesToString();	-- Setup default save strings

	_G["BBarSettingsFrameTab1"]:SetChecked(1);
	_G["BBarSettingsFrameTab2"]:SetChecked(nil);

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_LOGOUT");
	self:RegisterEvent("TIME_PLAYED_MSG");
	self:RegisterEvent("PLAYER_MONEY");
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
	self:RegisterEvent("UPDATE_FACTION");

	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA");

	self:RegisterEvent("PET_BATTLE_OPENING_START");
	self:RegisterEvent("PET_BATTLE_CLOSE");

end


function BBar_ChatFrame_DisplayTimePlayed(self, totalTime, levelTime)

	if(g_BBar.InfoCaptureTimeChat == 1) then 
		g_BBar.InfoCaptureTimeChat = 0;
		return;
	end
	oldChatFrame_DisplayTimePlayed(self, totalTime, levelTime);
end


function BBar_OnEvent(self, event, ...)

	if( event == "PET_BATTLE_OPENING_START" )	then	self:Hide();	end
	if( event == "PET_BATTLE_CLOSE" )			then	self:Show();	end
	if( event == "PLAYER_ENTERING_WORLD" ) then
		-- One time setup
		if(g_BBar.Setup == 1)	then	return	end;
		MoneyFrame_UpdateMoney(g_BBar.MoneyFrameObject);
		BBar_UpdateBar();
		BBar_UpdateMiniBars();

		BBar_BuildSettingsFrame();
		BBarTab_OnClick(1);		-- Setup the initial settings window
		g_BBar.CurentGold = GetMoney();
		g_BBar.Setup = 1;
	end
	if( event == "PLAYER_LOGOUT" ) then
		local BPs = {};
		BPs[1], BPs[2], BPs[3], BPs[4], BPs[5] = _G["BBarFrame"]:GetPoint();
		BBarPos = BPs;
	end
	if( event == "TIME_PLAYED_MSG" ) then

		if(g_BBar.InfoCaptureTimeWrite == 1) then 
			local arg1, arg2 = ...;
			local d, h, m, s = ChatFrame_TimeBreakDown(arg1);
			local text = format("%dd : %dh : %dm : %ds", math.floor(d), math.floor(h), math.floor(m), math.floor(s))
			_G["BBarInfoTotalTime"]:SetText("|cff40ff40Total Time Played|r  -  " .. text);
			d, h, m, s = ChatFrame_TimeBreakDown(arg2);
			text = format("%dd : %dh : %dm : %ds", math.floor(d), math.floor(h), math.floor(m), math.floor(s))
			_G["BBarInfoLevelTime"]:SetText("|cff40ff40At Current Level|r  -  " .. text);
			g_BBar.InfoCaptureTimeWrite = 0;
		end
	end
	if( event == "PLAYER_MONEY" ) then
		BBar_UpdateGold(GetMoney() - g_BBar.CurentGold);
		g_BBar.CurentGold = GetMoney();
		ValuesToString();	-- Store the updated amount

		MoneyFrame_UpdateMoney(g_BBar.MoneyFrameObject);
		BBar_UpdateBar();
	end
	if( event == "BAG_UPDATE" ) or ( event == "CURRENCY_DISPLAY_UPDATE" ) then
		BBar_UpdateBar();		-- Would be nice to know what changed on bag update so it only updates if needed
	end
	if( event == "UPDATE_FACTION" ) then
		BBar_UpdateMiniBars();		-- Would be nice to know what changed so it only updates if needed
	end
	if( event == "TRADE_SKILL_UPDATE" ) then
		BBar_UpdateMiniBars();		-- Would be nice to know what changed so it only updates if needed
	end
	
	if((event == "PLAYER_REGEN_DISABLED") and (g_BBarO.MinOnCombat == 1) and (g_BBar.rsCurrentSize == 1)) then
		g_BBarO.MinimiseState[1] = 1;
		BBar_ToggleSize();
	end
	if((event == "PLAYER_REGEN_ENABLED") and (g_BBarO.MinOnCombat == 1) and (g_BBarO.MinimiseState[1] == 1)) then
		g_BBarO.MinimiseState[1] = 0;
		BBar_ToggleSize();
	end

	if(event == "ZONE_CHANGED_NEW_AREA") then
	--	local name, type, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, mapID = GetInstanceInfo();
	--	_T("Info : "..name.."   Diff : "..difficultyIndex.."   Diff Name : "..difficultyName.."   Players : "..maxPlayers.."   Dyn Diff : "..dynamicDifficulty.."   Dynamic : "..tostring(isDynamic).."   MapID : "..tostring(mapID));
		local inInstance, zone = IsInInstance();
	--	_T("Inst : "..inInstance.."   Zone : "..zone);
		if(inInstance and (g_BBarO.MinOnInst == 1) and (g_BBar.rsCurrentSize == 1)) then
			g_BBarO.MinimiseState[2] = 1;
			BBar_ToggleSize();
		end
		if(not inInstance and (g_BBarO.MinOnInst == 1) and (g_BBarO.MinimiseState[2] == 1)) then
			g_BBarO.MinimiseState[2] = 0;
			BBar_ToggleSize();
		end
	end

	if( event == "VARIABLES_LOADED" ) then
		StringToValues();				-- parse variable string from storage
		g_BBar.OnetimeEventUpdate = 1;	-- Position the window (messy hack)
		BBar_ToggleMoving(nil);			-- Set up the window lock state
	end
end


function BBar_ToggleSettings()

	if(_G["BBarSettingsFrame"]:IsShown()) then
			HideUIPanel(_G["BBarSettingsFrame"]);
	else	ShowUIPanel(_G["BBarSettingsFrame"]);
	end
end


function BBar_ToggleSize()

	-- Large -> Small
	if(g_BBar.rsCurrentSize == 1) then
		_G["BBarFrameResizing"]:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
		_G["BBarFrameResizing"]:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");

		g_BBar.rsStoredWidth = _G["BBarFrame"]:GetWidth();	-- Store the current width
		g_BBar.rsAnimatingSize = g_BBar.rsStoredWidth;

		-- Hide everything
		_G["BBarFrameMoneyFrame"]:Hide();
		_G["BBarFrameSettings"]:Hide();
		
		-- Currencies
		local i = 3;	-- Start on 3 to skip the money frame
		while (g_BBar.BBarObjects[i] ~= nil) do
			if(g_BBar.BBarObjects[i] > g_BBar.MINCURRENCYID) then
				BBar_AddNewCurrencyToken(0, 0, g_BBar.BBarObjects[i], 0);
			end
			i = i + 1;
		end

		-- Custom items
		for i = 1, getn(g_BBar.BBarObjectsItems), 1 do
			BBar_AddNewCurrencyToken(0, 0, -g_BBar.BBarObjectsItems[i], 0);
		end

		-- Reputations
		for pos, values in pairs(g_BBar.BBarObjectsReps_Faction) do
			BBar_AddNewReputationBar(0, 0, values[1], 0, 0);
		end

		-- Professions
		local professions = {GetProfessions()}
		local i = 1;
		for _, v in pairs(professions) do
			if v then
				BBar_AddNewProfessionBar(0, 0, i, 0, 0, 0, 0, 0);
				i = i + 1;	-- Move to the next one
			end
		end

	-- Small -> Large
	else
		_G["BBarFrameResizing"]:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");
		_G["BBarFrameResizing"]:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");
		g_BBar.rsAnimatingSize = g_BBar.BASEBARMINSZ;
		g_BBarO.MinimiseState = {0, 0};		-- Reset both back to default
		-- When animation is finished, it'll update the bar to draw all currencies and mini bars
	end

	-- Start the resize
	g_BBar.rsCurrentSize = (1 - g_BBar.rsCurrentSize);
	g_BBar.rsAnimating = 1;	-- Start animating
end


function BBar_UpdateEvent(self)

	-- Update the window
	if(g_BBar.OnetimeEventUpdate == 1) then
		if(BBarPos ~= nil and BBarPos[1] ~= nil)	then
			_G["BBarFrame"]:ClearAllPoints();
			_G["BBarFrame"]:SetPoint(BBarPos[1], BBarPos[2], BBarPos[3], BBarPos[4], BBarPos[5]);
		end
		g_BBar.OnetimeEventUpdate = 0;
	end
	BBar_ReputationTooltipUpdate();

	-- If not animating then skip the rest
	if(g_BBar.rsAnimating ~= 1)	then	return	end;

	StoreBBPos();
	-- Growing
	if(g_BBar.rsCurrentSize == 1) then
		if(g_BBar.rsAnimatingSize >= g_BBar.rsStoredWidth) then
			self:SetWidth(g_BBar.rsStoredWidth);
			g_BBar.rsAnimating = 0;	-- Stop animation
			_G["BBarFrameSettings"]:Show();
			BBar_UpdateBar();	-- Make sure it's all as it should be
		else
			g_BBar.rsAnimatingSize = g_BBar.rsAnimatingSize + g_BBar.BARRESIZESTEP;
			if(g_BBar.rsAnimatingSize > g_BBar.rsStoredWidth) then	g_BBar.rsAnimatingSize = g_BBar.rsStoredWidth;	end
			self:SetWidth(g_BBar.rsAnimatingSize);
		end
	else	-- Shrinking
		if(g_BBar.rsAnimatingSize <= g_BBar.BASEBARMINSZ) then
			self:SetWidth(g_BBar.BASEBARMINSZ);
			g_BBar.rsAnimating = 0;	-- Stop animation
		else
			g_BBar.rsAnimatingSize = g_BBar.rsAnimatingSize - g_BBar.BARRESIZESTEP;
			if(g_BBar.rsAnimatingSize < g_BBar.BARRESIZESTEP) then	g_BBar.rsAnimatingSize = g_BBar.BARRESIZESTEP;	end
			self:SetWidth(g_BBar.rsAnimatingSize);
		end
	end
	UpdateBBPos();
end


function BBar_ShowCurrencyTooltip(self)
	if((self == nil) or (self.currencyID == nil)) then	return	end
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT");
	if(self.currencyID > g_BBar.MINCURRENCYID)	then	GameTooltip:SetCurrencyByID(self.currencyID);	end
	if(self.currencyID < 0)						then	GameTooltip:SetItemByID(abs(self.currencyID));	end
end


function BBar_ShowResizeTooltip(self)

	GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT");
	if(g_BBar.rsCurrentSize == 1) then	GameTooltip:SetText("Shrink Bar", 1, 1, 1);
	else	GameTooltip:SetText("Enlarge Bar", 1, 1, 1);
	end
	GameTooltip:Show();
end


function BBar_ReputationTooltipUpdate()

	if(g_BBar.UpdateTooltip == 1) then
		local scale = UIParent:GetScale();
		local x, y = GetCursorPosition()
		x = x / scale;
		y = y / scale;
		GameTooltip:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", x - 4, y + 4);
	end
end

function BBar_ShowReputationTooltip(self)

	local name =		_G[self:GetName().."FactionName"]:GetText();
	local value =		_G[self:GetName().."Value"]:GetText();
	local standing =	_G[self:GetName().."Standing"]:GetText();

	GameTooltip:SetOwner(self, "ANCHOR_NONE");
	GameTooltip:SetText(name, 1, 1, 1);
	GameTooltip:AddLine(value, 1, 1, 1);
	GameTooltip:AddLine(standing, 1, 1, 1);
	GameTooltip:Show();
	g_BBar.UpdateTooltip = 1;
end

function BBar_HideReputationTooltip(self)

	g_BBar.UpdateTooltip = 0;
	GameTooltip:Hide();
end


---------------------------


function BBar_UpdateBar()

	if(g_BBar.rsCurrentSize == 0)		then	return;			end		-- Skip when minimised
	if(g_BBar.UpdateBarPosition == 1)	then	StoreBBPos();	end

	-- Add new tokens
	local priorObj = nil;
	local extraWidth = 0;
	local self = _G["BBarFrame"];

	if((g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] ~= nil) and (g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] > 0)) then
			_G["BBarFrameMoneyFrame"]:Show();
	else	_G["BBarFrameMoneyFrame"]:Hide();
	end

	-- Currencies
	local i = 3;	-- Start on 3 to skip the money frame
	while (g_BBar.BBarObjects[i] ~= nil) do
		if(g_BBar.BBarObjects[i] > g_BBar.MINCURRENCYID) then
			local outWidth, outObj = BBar_AddNewCurrencyToken(self, priorObj, g_BBar.BBarObjects[i], g_BBar.BBarObjects_Shown[i]);
			if(outWidth > 0)	then	extraWidth = extraWidth + outWidth;		end
			if(outObj ~= 0)		then	priorObj = outObj;						end
		end
		i = i + 1;
	end

	-- Custom items
	for i = 1, getn(g_BBar.BBarObjectsItems), 1 do
		local outWidth, outObj = BBar_AddNewCurrencyToken(self, priorObj, -g_BBar.BBarObjectsItems[i], g_BBar.BBarObjectsItems_Shown[i]);
		if(outWidth > 0)	then	extraWidth = extraWidth + outWidth;		end
		if(outObj ~= 0)		then	priorObj = outObj;						end
	end
	
	local moneyWidth = 0;
	if((g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] ~= nil) and (g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] > 0))
	then	moneyWidth = _G["BBarFrameMoneyFrame"]:GetWidth();	end
	local TotalWidth = g_BBar.BASEBARSZ + moneyWidth + extraWidth;
	self:SetWidth(TotalWidth);

	BBar_UpdateMiniBars();

	if(g_BBar.UpdateBarPosition == 1)	then	UpdateBBPos();	end
	g_BBar.UpdateBarPosition = 1;	-- Make it update on all subsequent passes
end


function BBar_UpdateMiniBars()

	if(g_BBar.rsCurrentSize == 0)		then	return;			end		-- Skip when minimised

	local bar = _G["BBarFrame"];
	local TotalWidth = (bar:GetWidth() - g_BBar.BARREPSHIFT);	-- Remove the spacing on the left 
	local priorObj = nil;

-- Update the reputation bars
	-- Because of dodgy lua i need to extract the data and sort the list manually
	local count = 0;
	local tempStore = {};
	for pos, values in pairs(g_BBar.BBarObjectsReps_Faction) do
		tinsert(tempStore, {pos, values[1], values[2]});
		count = count + 1;
	end
	if(#tempStore > 1)	then	sort(tempStore, function(a, b) return a[1] < b[1] end);	end;

	for i = 1, count do
		local outObj = BBar_AddNewReputationBar(bar, priorObj, tempStore[i][2], tempStore[i][3], TotalWidth);
		if(outObj ~= 0)	then	priorObj = outObj;	end
	end

-- Update the profession bars
	local professions = {GetProfessions()}
	local i = 1;
	for _, v in pairs(professions) do
		if v then
			local name, icon, rank, maxRank, numSpells, spelloffset, skillLine = GetProfessionInfo(v);
			local outObj = BBar_AddNewProfessionBar(bar, priorObj, i, spelloffset, name, rank, maxRank, g_BBar.BBarObjectsProfs_Shown[i], TotalWidth);
			if(outObj ~= 0)	then	priorObj = outObj;	end
			i = i + 1;	-- Move to the next one
		end
	end
end


function BBar_AddNewCurrencyToken(self, priorObj, currencyID, shown)

	local thisObj = _G["BBarCurrencyToken"..currencyID];

	-- Check if this button should be made or shown
	if(shown == 0) then
		if(thisObj) then thisObj:Hide(); end
		return 0, 0;
	end

	-- If this button doesn't exist yet, create it
	if( not thisObj ) then
		thisObj = CreateFrame("BUTTON", "BBarCurrencyToken"..currencyID, self, "BBarCurrencyFrameTemplate");
	else	thisObj:ClearAllPoints();
	end
	_G[thisObj:GetName().."Spacer"]:Show();		-- Incase it was hidden by being the first item earlier

	-- Get this currency info and update the button
	local objWidth = 0;
	local name, count, icon;
	if(currencyID > g_BBar.MINCURRENCYID) then
		name, count, icon = GetCurrencyInfo(currencyID);
	elseif(currencyID < 0) then		-- Item
		name = GetItemInfo(abs(currencyID));
		count = GetItemCount(abs(currencyID), true);
		icon = GetItemIcon(abs(currencyID));
	end

	if( name and name ~= "" ) then
		thisObj:SetText(count);
		thisObj:SetNormalTexture(icon);
		thisObj.currencyID = currencyID;
		thisObj:Show();
		objWidth = thisObj:GetTextWidth() + thisObj:GetWidth() + 28;
		if(priorObj ~= nil)	then	thisObj:SetPoint("LEFT", priorObj, "RIGHT", priorObj:GetTextWidth() + 28, 0);
		else
			if((g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] ~= nil) and (g_BBar.BBarObjects_Shown[g_BBar.MONEYPOS] > 0)) then
				thisObj:SetPoint("LEFT", _G["BBarFrameMoneyFrame"], "RIGHT", 13, 0);
			else
				_G[thisObj:GetName().."Spacer"]:Hide();
				thisObj:SetPoint("LEFT", _G["BBarFrame"], "LEFT", 48, -1);
			end
		end
	else	thisObj:Hide();	end
	return objWidth, thisObj;
end


function BBar_AddNewProfessionBar(self, priorObj, profID, profOffset, name, rank, maxRank, shown, width)

	local thisObj = _G["BBarProfessionFrame"..profID];

	-- Check if this frame should be made or shown
	if(shown == 0) then
		if(thisObj) then thisObj:Hide(); end
		return 0;
	end

	-- If this frame doesn't exist yet, create it
	if( not thisObj ) then
		thisObj = CreateFrame("FRAME", "BBarProfessionFrame"..profID, self, "BBarReputationFrameTemplate");
	else	thisObj:ClearAllPoints();
	end
	thisObj:SetWidth(width);
	thisObj:SetHeight(g_BBarO.SliderValues[3]);
	thisObj.barType = 2;	-- Profession
	thisObj:SetID(profOffset);
--	_T(tostring(width));
	local factionTitle =		_G["BBarProfessionFrame"..profID.."FactionName"];
	local factionValue =		_G["BBarProfessionFrame"..profID.."Value"];
	local factionStanding =		_G["BBarProfessionFrame"..profID.."Standing"];
	local factionBarFill =		_G["BBarProfessionFrame"..profID.."PercentageBar"];
	factionValue:Hide();	-- Unused
	factionTitle:SetHeight(g_BBarO.SliderValues[3] - 2);
	factionStanding:SetHeight(g_BBarO.SliderValues[3] - 2);
	factionBarFill:SetHeight(g_BBarO.SliderValues[3] - 2);
	factionTitle:SetTextColor(1, 1, 1, g_BBarO.SliderValues[4]);
	factionStanding:SetTextColor(1, 1, 1, g_BBarO.SliderValues[4]);

	-- Hide things if not enough space
	if(width < 200) then	factionStanding:Hide();
	else					factionStanding:Show();
	end

	if(rank == 0) then
		factionBarFill:Hide();	-- Supplying 0 causes max size
	else
		factionBarFill:Show();
		factionBarFill:SetWidth((((width - 2) / abs(maxRank)) * abs(rank)) - 1);
	end

	factionTitle:SetText(name);
	factionStanding:SetText(tostring(rank).."/"..tostring(maxRank));
	factionBarFill:SetVertexColor(0.1, 1.0, 0.1, 0.8);
	thisObj:Show();

	-- Position
	if(priorObj ~= nil)	then	thisObj:SetPoint("TOPLEFT", priorObj, "BOTTOMLEFT", 0, 1);
	else						thisObj:SetPoint("TOPLEFT", _G["BBarFrame"], "BOTTOMLEFT", g_BBar.BARREPSHIFT, 1);
	end
	return thisObj;
end


function BBar_AddNewReputationBar(self, priorObj, factionID, shown, width)

	local thisObj = _G["BBarReputationFrame"..factionID];

	-- Check if this frame should be made or shown
	if(shown == 0) then
		if(thisObj) then thisObj:Hide(); end
		return 0;
	end

	-- If this frame doesn't exist yet, create it
	if( not thisObj ) then
		thisObj = CreateFrame("FRAME", "BBarReputationFrame"..factionID, self, "BBarReputationFrameTemplate");
	else	thisObj:ClearAllPoints();
	end
	thisObj:SetWidth(width);
	thisObj:SetHeight(g_BBarO.SliderValues[1]);
	thisObj.barType = 1;	-- Reputation
	local factionTitle =		_G["BBarReputationFrame"..factionID.."FactionName"];
	local factionValue =		_G["BBarReputationFrame"..factionID.."Value"];
	local factionStanding =		_G["BBarReputationFrame"..factionID.."Standing"];
	local factionBarFill =		_G["BBarReputationFrame"..factionID.."PercentageBar"];
	factionTitle:SetHeight(g_BBarO.SliderValues[1] - 2);
	factionValue:SetHeight(g_BBarO.SliderValues[1] - 2);
	factionStanding:SetHeight(g_BBarO.SliderValues[1] - 2);
	factionBarFill:SetHeight(g_BBarO.SliderValues[1] - 2);
	factionTitle:SetTextColor(1, 1, 1, g_BBarO.SliderValues[2]);
	factionValue:SetTextColor(1, 1, 1, g_BBarO.SliderValues[2]);
	factionStanding:SetTextColor(1, 1, 1, g_BBarO.SliderValues[2]);

	-- Hide things if not enough space
	if(width < 370) then	factionValue:Hide();
	else					factionValue:Show();
	end
	if(width < 200) then	factionStanding:Hide();
	else					factionStanding:Show();
	end

	-- Get this faction info and update the frame
	local name, _, standingID, barMin, barMax, barValue = GetFactionInfoByID(factionID);
	factionTitle:SetText(name);
	local factionStandingtext = GetText("FACTION_STANDING_LABEL"..standingID, UnitSex("player"));

	-- Check if this is a friendship faction (members of the Tillers)
	local friendID, _, friendMaxRep, _, _, _, friendTextLevel, friendThresh = GetFriendshipReputation(factionID);
	if(friendID ~= nil) then
		factionStandingtext = friendTextLevel;
		barMax = friendMaxRep - friendThresh;
		barValue = barValue - friendThresh;
	else
		barMax = barMax - barMin;
		barValue = barValue - barMin;
		if(barValue < 0) then	barValue = (barMax - barValue);	end
	end
	if(barValue == 0) then
			factionBarFill:Hide();	-- Supplying 0 causes max size
	else
		factionBarFill:Show();
		factionBarFill:SetWidth((((width - 2) / abs(barMax)) * abs(barValue)) - 1);
	end

	factionValue:SetText(tostring(barValue).."/"..tostring(barMax));
	factionStanding:SetText(factionStandingtext);
	local color = FACTION_BAR_COLORS[standingID];
	factionBarFill:SetVertexColor(color.r, color.g, color.b, 0.8);
	thisObj:Show();

	-- Position
	if(g_BBarO.BarGrowDir == 1) then	-- Upwards
		if(priorObj ~= nil)	then	thisObj:SetPoint("BOTTOMLEFT", priorObj, "TOPLEFT", 0, -1);
		else						thisObj:SetPoint("BOTTOMLEFT", _G["BBarFrame"], "TOPLEFT", g_BBar.BARREPSHIFT, -1);
		end
	else	-- Downwards
		if(priorObj ~= nil)	then	thisObj:SetPoint("TOPLEFT", priorObj, "BOTTOMLEFT", 0, 1);
		else						thisObj:SetPoint("TOPLEFT", _G["BBarFrame"], "BOTTOMLEFT", g_BBar.BARREPSHIFT, 1);
		end
	end
	return thisObj;
end


---------------------------


-- BBar location storage during resizing.
function StoreBBPos()

	-- Get the location so we can keep it left aligned when resized.
	g_BBar.PosStorage[1], g_BBar.PosStorage[2], g_BBar.PosStorage[3], g_BBar.PosStorage[4], g_BBar.PosStorage[5] = _G["BBarFrame"]:GetPoint();
	g_BBar.PosStorage[6] = _G["BBarFrame"]:GetWidth();
end

function UpdateBBPos()

	-- Only update if it was stored correctly
	if(g_BBar.PosStorage[1] == nil) then	return	end;

	local Width = _G["BBarFrame"]:GetWidth();
	
	if(strfind(g_BBar.PosStorage[1], "LEFT") ~= nil) then
		g_BBar.PosStorage[4] = g_BBar.PosStorage[4];		-- Do nothing
	elseif(strfind(g_BBar.PosStorage[1], "RIGHT") ~= nil) then
		g_BBar.PosStorage[4] = (g_BBar.PosStorage[4] - g_BBar.PosStorage[6]) + Width;
	else	-- TOP CENTER BOTTOM
		g_BBar.PosStorage[4] = (g_BBar.PosStorage[4] - (g_BBar.PosStorage[6] / 2)) + (Width / 2);
	end
	_G["BBarFrame"]:SetPoint(g_BBar.PosStorage[1], g_BBar.PosStorage[2], g_BBar.PosStorage[3], g_BBar.PosStorage[4], g_BBar.PosStorage[5]);
	g_BBar.PosStorage[1] = nil;
end

