--[[

	Information reporting
	Item Loading/Saving
	Extra helper functions

]]--

-- Report style and layout from Recount
local BBar_ReportLocations = {
	{"Say", "SAY"},
	{"Party", "PARTY", function() return (GetNumPartyMembers() > 0) end},
	{"Instance", "INSTANCE_CHAT", function() return (GetNumGroupMembers(LE_PARTY_CATEGORY_INSTANCE) ~= 0) end},
	{"Raid", "RAID", function() return UnitInRaid("player") end},
	{"Guild", "GUILD", IsInGuild},
	{"Officer", "OFFICER", IsInGuild},
	{"Whisper Target", "WHISPER"},
	{"Battle Net Chat", "BN_WHISPER"}
}


function BBar_BNMenuShow(parent, menu)
	menu:ClearAllPoints();
	menu:SetPoint("TOPLEFT", parent, "TOPRIGHT", 0, 8);
	menu:Show();
end


function BBar_AddChannelMenuItem(buttonArray, parent, text, id, pos, source, chatType, channel)

	local width = 0;
	local item = _G["BBarChannelMenuItemT"..id];
	if(not item) then
		item = parent:CreateFontString("BBarChannelMenuItemT"..id, nil, "GameFontHighlightSmall")
		item:SetHeight(20);
		item:SetJustifyH("LEFT")
	end
	if(chatType ~= "VOID") then		-- Not disabled
		local button = _G["BBarChannelMenuItemB"..id];
		if(not button) then
			button = CreateFrame("Button", "BBarChannelMenuItemB"..id, parent);
			button:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight");
			button:GetHighlightTexture():SetBlendMode("ADD");
			button:GetHighlightTexture():SetVertexColor(1, 1, 1, 0.8);
			button:SetHeight(20);
			button:RegisterForClicks("LeftButtonUp");
		end
		tinsert(buttonArray, button);	-- Store for later
		button:ClearAllPoints();
		button:SetPoint("TOPLEFT", parent, "TOPLEFT", 5, -pos);
		button:Show();
		if(id == 99)	then
			button:SetScript("OnClick", function() parent:Hide() end);
		elseif(chatType ~= "VOID") then
			button:SetScript("OnClick", function() BBar_AddChatInfo(source, chatType, channel) end);
		end
		item:SetTextColor(1.0, 1.0, 1.0, 1);

		local BNmenu = _G["BBarChannelMenuBN"];
		if(parent ~= BNmenu) then button:SetScript("OnEnter", function() BNmenu:Hide() end); end

		-- Submenu extras
		if((chatType == "BN_WHISPER") and (channel == 0)) then
			local texture = _G["BBarChannelMenuBNTex"];
			if( not texture ) then
				texture = parent:CreateTexture(name);
			else	texture:ClearAllPoints();
			end
			texture:SetTexture("Interface\\ChatFrame\\ChatFrameExpandArrow");
			texture:SetDesaturated(true);
			texture:SetWidth(10);
			texture:SetPoint("RIGHT", button, "RIGHT", -5, 0);
			width = width + 12;
			button:SetScript("OnEnter", function() BBar_BNMenuShow(button, BNmenu) end);
		end
	else
		item:SetTextColor(0.6, 0.6, 0.6, 1);
	end

	item:ClearAllPoints();
	item:SetPoint("TOPLEFT", parent, "TOPLEFT", 10, -pos);
	item:SetText(text);
	item:Show();
	width = width + item:GetWidth();
	return (pos + 20), width;
end


function BBar_DisplayChannelMenu(self, button)

	if (button ~= "RightButton") then return; end
	GameTooltip:Hide();

-- Create/Fill friends menu

	local BNmenu = _G["BBarChannelMenuBN"];	-- Battle net friends list menu
	if(not BNmenu) then
		BNmenu = CreateFrame("Frame", "BBarChannelMenuBN", UIParent, "BBarSettingsRaidTemplate");
		BNmenu:SetFrameStrata("TOOLTIP");
		BNmenu:SetSize(200, 8);
		BNmenu:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=true, tileSize=16, edgeSize=10, insets={left=2, right=2, top=2, bottom=2}});
		BNmenu:SetBackdropBorderColor(1, 1, 1, 1);
		BNmenu:SetBackdropColor(0, 0, 0, 1);
	else	BNmenu:ClearAllPoints();
	end

	local buttonArray = {};
	local id = 100;		-- Start at 100 for the friend menu
	local width = 0;
	local maxWidth = 0;
	local pos = 8;	-- Padding of 8 at the top

	local numBNetTotal, numBNetOnline = BNGetNumFriends(); -- FRIENDS_BUTTON_TYPE_BNET
	local numWoWTotal, numWoWOnline = GetNumFriends();	-- FRIENDS_BUTTON_TYPE_WOW
	local numBNetOffline = numBNetTotal - numBNetOnline;
	local numWoWOffline = numWoWTotal - numWoWOnline;
	local totalFriends = (numBNetTotal + numWoWTotal);
	if(totalFriends > 0) then

		local line = nil;
		local index = 1;
		-- Online friends
		for i = 1, numBNetOnline do		-- online BN friends
			local presenceID, givenName, surname, _, _, _, isOnline = BNGetFriendInfo(index);
			index = index + 1;
			pos, width = BBar_AddChannelMenuItem(buttonArray, BNmenu, givenName, id, pos, self, "BN_WHISPER", presenceID);
			if(width > maxWidth) then maxWidth = width; end
			id = id + 1;
		end
		for i = 1, numWoWOnline do		-- online WoW friends
			local name, _, _, _, isOnline = GetFriendInfo(index);
			index = index + 1;
			pos, width = BBar_AddChannelMenuItem(buttonArray, BNmenu, name, id, pos, self, "WHISPER2", name);
			if(width > maxWidth) then maxWidth = width; end
			id = id + 1;
		end
		if((numBNetOnline > 0) or (numWoWOnline > 0)) then
			line = BBar_CreateDrawnLine("BBar_ReportMenuBNLine", BNmenu, 0, -(pos + 4), 10);
			pos = pos + 8;
		end

		-- Offline friends (greyed)
		for i = 1, numBNetOffline do		-- online BN friends
			local presenceID, givenName, surname, _, _, _, isOnline = BNGetFriendInfo(index);
			index = index + 1;
			local name = string.format(BATTLENET_NAME_FORMAT, givenName, surname);
			pos, width = BBar_AddChannelMenuItem(buttonArray, BNmenu, givenName, id, pos, self, "VOID", 0);
			if(width > maxWidth) then maxWidth = width; end
			id = id + 1;
		end
		for i = 1, numWoWOffline do		-- online WoW friends
			local name, _, _, _, isOnline = GetFriendInfo(index);
			index = index + 1;
			pos, width = BBar_AddChannelMenuItem(buttonArray, BNmenu, name, id, pos, self, "VOID", 0);
			if(width > maxWidth) then maxWidth = width; end
			id = id + 1;
		end
		BNmenu:SetHeight(pos + 8);	-- Padding of 8 at the bottom

		-- Update the window width to the max string size
		BNmenu:SetWidth(maxWidth + 20);
		if(line) then line:SetWidth(maxWidth); end
		for _,v in ipairs(buttonArray) do	v:SetWidth(maxWidth + 12);	end
	end

-- Create/Fill main menu

	local menu = _G["BBarChannelMenu"];
	if( not menu ) then
		menu = CreateFrame("Frame", "BBarChannelMenu", UIParent, "BBarSettingsRaidTemplate");
		menu:SetFrameStrata("TOOLTIP");
		menu:SetSize(200, 8);
		menu:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=true, tileSize=16, edgeSize=10, insets={left=2, right=2, top=2, bottom=2}});
		menu:SetBackdropBorderColor(1, 1, 1, 1);
		menu:SetBackdropColor(0, 0, 0, 1);
	else	menu:ClearAllPoints();
	end

	-- Fill the menu
	buttonArray = {};
	id = 1;
	width = 0;
	maxWidth = 0;
	pos = 8;	-- Padding of 8 at the top
	for _,v in ipairs(BBar_ReportLocations) do
		if ((type(v[3]) ~= "function") or v[3]()) then
			if((v[2] == "BN_WHISPER") and (totalFriends == 0)) then v[2] = "VOID"; end	-- If no friends disable the option.
			pos, width = BBar_AddChannelMenuItem(buttonArray, menu, v[1], id, pos, self, v[2], 0);
			if(width > maxWidth) then maxWidth = width; end
			id = id + 1;
		end
	end

	local line1 = BBar_CreateDrawnLine("BBar_ReportMenuLine1", menu, 0, -(pos + 4), 10);
--	line1:SetScript("OnEnter", function() BNmenu:Hide() end);
	pos = pos + 8;

	local channels = {GetChannelList()}
	for i = 1, table.getn(channels) / 2 do
		pos, width = BBar_AddChannelMenuItem(buttonArray, menu, tostring(channels[(i * 2) - 1])..". "..tostring(channels[i * 2]), id, pos, self, "CHANNEL", channels[(i * 2) - 1]);
		if(width > maxWidth) then maxWidth = width; end
		id = id + 1;
	end

	-- Scroll through any other created raid item frames to hide them
	for i = id, 20, 1 do
		local item = _G["BBarChannelMenuItemB"..i];		if(item) then item:Hide(); end
		local item = _G["BBarChannelMenuItemT"..i];		if(item) then item:Hide(); end
	end
	local line2 = BBar_CreateDrawnLine("BBar_ReportMenuLine2", menu, 0, -(pos + 4), 10);
--	line2:SetScript("OnEnter", function() BNmenu:Hide() end);
	pos = pos + 8;
	pos, width = BBar_AddChannelMenuItem(buttonArray, menu, "Cancel", 99, pos, nil, nil, 0);		-- Using 99 as the cancel ID
	if(width > maxWidth) then maxWidth = width; end
	menu:SetHeight(pos + 8);	-- Padding of 8 at the bottom

	-- Update the window width to the max string size
	menu:SetWidth(maxWidth + 20);
	line1:SetWidth(maxWidth);
	line2:SetWidth(maxWidth);
	for _,v in ipairs(buttonArray) do	v:SetWidth(maxWidth + 12);	end

	-- Position at the cursor and show
	local scale = UIParent:GetScale();
	local x, y = GetCursorPosition()
	x = x / scale;
	y = y / scale;
--	_T("Pos X:"..x.."  Pos Y:"..y);
	menu:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", x - 4, y + 4);
	menu:Show();
end


-----------------------------------------------


function BBar_AddChatInfo(source, chatType, Channel)

	local menu = _G["BBarChannelMenu"];
	if(menu) then	menu:Hide();	end
	local BNmenu = _G["BBarChannelMenuBN"];
	if(BNmenu) then	BNmenu:Hide();	end

	local Title = "";
	local Value = "";
	local Standing = "";
	local text = "";
	if(chatType == "WHISPER") then
		if(UnitExists("target")) then
			if(UnitIsPlayer("target")) then
				Channel = UnitName("target");
			else
				_T("Whisper Failed, Target isn't a Player", 1, 0.1, 0.1);
				return;
			end
		else
			_T("Whisper Failed, No Target Selected", 1, 0.1, 0.1);
			return;
		end
	end
	if(chatType == "WHISPER2") then	chatType = "WHISPER" end	-- Channel already has the name in it.

	if(source.barType == 10) then	-- Settings all instances
		local numInstances = GetNumSavedInstances();
		for i = 1, numInstances, 1 do
			local name, id, _, _, locked, _, _, _, _, _, numEncounters, encounterProgress = GetSavedInstanceInfo(i);
			if(locked == true) then
				Value =	tostring(encounterProgress).."/"..tostring(numEncounters);
				text = "{Triangle} "..name.." ("..Value.. ") - id:"..tostring(id).." {Triangle}";
				if(chatType == "BN_WHISPER") then	BNSendWhisper(Channel, text);	-- Channel is ID
				else	SendChatMessage(text, chatType, nil, Channel);
				end
			end
		end
		return;
	end

	if(source.barType == 1 or source.barType == 2) then
		Title =		_G[source:GetName().."FactionName"]:GetText();
		Value =		_G[source:GetName().."Value"]:GetText();
		Standing =	_G[source:GetName().."Standing"]:GetText();

		if(source.barType == 1) then	-- Reputation
			text = "{Star} "..Title.." - "..Value.. " ("..Standing..") {Star}";
		end
		if(source.barType == 2) then	-- Profession
			local spellLink, tradeSkillLink = GetSpellLink(source:GetID() + 1, "professions");
			if(tradeSkillLink) then	Title = tradeSkillLink;	end
			text = "{Diamond} "..Title.." - "..Standing.." {Diamond}";
		end
	end
	if(source.barType == 3) then	-- Settings reputations
		Title =	_G[source:GetName().."FactionName"]:GetText();
		local factionBar = _G[source:GetName().."RepBar"];
		text = "{Star} "..Title.." - "..factionBar.reptext.. " ("..factionBar.standingtext..") {Star}";
	end
	if(source.barType == 4) then	-- Settings instances
		Title =	_G[source:GetName().."Name"]:GetText();
		Value =	_G[source:GetName().."Progress"]:GetText();
		text = "{Triangle} "..Title.." ("..Value.. ") - id:"..source.instID.." {Triangle}";
	end
	if(source.barType == 5) then	-- Settings professions
		Title =		_G[source:GetName().."Name"]:GetText();
		Standing =	_G[source:GetName().."BarValue"]:GetText();
		local spellLink, tradeSkillLink = GetSpellLink(source:GetID() + 1, "professions");
		if(tradeSkillLink) then	Title = tradeSkillLink;	end
		text = "{Diamond} "..Title.." - "..Standing.." {Diamond}";
	end
	if(chatType == "BN_WHISPER") then	BNSendWhisper(Channel, text);	-- Channel is ID
	else	SendChatMessage(text, chatType, nil, Channel);
	end
end


function BBar_AddChatString(text)

	if(IsModifiedClick("CHATLINK")) then
		local activeWindow = ChatEdit_GetActiveWindow();
	--	activeWindow:SetMaxBytes(512);
	--	activeWindow:SetMaxLetters(512);
	--	_T(tostring(activeWindow:GetMaxLetters()));
	--	_T(tostring(activeWindow:GetMaxBytes()));
		
		if(activeWindow) then	-- Construct the string
			activeWindow:Insert(" "..text);	-- add a space for proper parsing
		end
	end
end


--------------------------------------------------------


function BBar_UpdateGold(gold)

	-- gold is the amount changed, can be 0 if tooltip update check
	local weekday = CalendarGetDate();
	if(weekday ~= g_BBar.CurrentDay) then
		if(weekday == 1) then		-- New week
			g_BBar.GoldLastWeek[1] = g_BBar.GoldThisWeek[1];
			g_BBar.GoldLastWeek[2] = g_BBar.GoldThisWeek[2];
			g_BBar.GoldThisWeek = {0, 0};
		end
		g_BBar.GoldYesterday[1] = g_BBar.GoldToday[1];
		g_BBar.GoldYesterday[2] = g_BBar.GoldToday[2];
		g_BBar.GoldToday = {0, 0};
		g_BBar.CurrentDay = weekday;
	end

	-- Update values
	if(gold > 0) then
		g_BBar.GoldSession[1] = g_BBar.GoldSession[1] + gold;
		g_BBar.GoldToday[1] = g_BBar.GoldToday[1] + gold;
		g_BBar.GoldThisWeek[1] = g_BBar.GoldThisWeek[1] + gold;
	end
	if(gold < 0) then
		g_BBar.GoldSession[2] = g_BBar.GoldSession[2] + abs(gold);
		g_BBar.GoldToday[2] = g_BBar.GoldToday[2] + abs(gold);
		g_BBar.GoldThisWeek[2] = g_BBar.GoldThisWeek[2] + abs(gold);
	end
end


function BBar_ShowGoldTooltip(self)
	BBar_UpdateGold(0);
--	_G["BBarMoneyToolTipSessionGain"]:SetText(BBar_CreateGoldTooltipText(123456789000), 0);
--	_G["BBarMoneyToolTipSessionLost"]:SetText(BBar_CreateGoldTooltipText(123456789000), 0);
--	_G["BBarMoneyToolTipSessionTotal"]:SetText(BBar_CreateGoldTooltipText(123456789000, 1));
	_G["BBarMoneyToolTipSessionGain"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldSession[1]), 0);
	_G["BBarMoneyToolTipSessionLost"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldSession[2]), 0);
	_G["BBarMoneyToolTipSessionTotal"]:SetText(BBar_CreateGoldTooltipText((g_BBar.GoldSession[1] - g_BBar.GoldSession[2]), 1));
	_G["BBarMoneyToolTipTodayGain"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldToday[1]), 0);
	_G["BBarMoneyToolTipTodayLost"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldToday[2]), 0);
	_G["BBarMoneyToolTipTodayTotal"]:SetText(BBar_CreateGoldTooltipText((g_BBar.GoldToday[1] - g_BBar.GoldToday[2]), 1));
	_G["BBarMoneyToolTipYesterdayGain"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldYesterday[1]), 0);
	_G["BBarMoneyToolTipYesterdayLost"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldYesterday[2]), 0);
	_G["BBarMoneyToolTipYesterdayTotal"]:SetText(BBar_CreateGoldTooltipText((g_BBar.GoldYesterday[1] - g_BBar.GoldYesterday[2]), 1));
	_G["BBarMoneyToolTipThisWeekGain"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldThisWeek[1]), 0);
	_G["BBarMoneyToolTipThisWeekLost"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldThisWeek[2]), 0);
	_G["BBarMoneyToolTipThisWeekTotal"]:SetText(BBar_CreateGoldTooltipText((g_BBar.GoldThisWeek[1] - g_BBar.GoldThisWeek[2]), 1));
	_G["BBarMoneyToolTipLastWeekGain"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldLastWeek[1]), 0);
	_G["BBarMoneyToolTipLastWeekLost"]:SetText(BBar_CreateGoldTooltipText(g_BBar.GoldLastWeek[2]), 0);
	_G["BBarMoneyToolTipLastWeekTotal"]:SetText(BBar_CreateGoldTooltipText((g_BBar.GoldLastWeek[1] - g_BBar.GoldLastWeek[2]), 1));

	BBarMoneyToolTip:ClearAllPoints();
	BBarMoneyToolTip:SetPoint("TOPLEFT", self, "BOTTOMRIGHT", -30, -6);
	BBarMoneyToolTip:Show();
end


function BBar_CreateGoldTooltipText(money, layout)

	local text = "";
	if(money == 0)	then	text = "|cffffffff- - -|r";
	else
		local gold = floor(abs(money) / (COPPER_PER_GOLD));
		local goldtext = BreakUpLargeNumbers(gold);
		local silver = floor((abs(money) - (gold * COPPER_PER_GOLD)) / COPPER_PER_SILVER);
		local copper = mod(abs(money), COPPER_PER_SILVER);
		local force = false;
		if(layout == 1) then	text = _C((money < 0), "|cffff1010- |r", "|cff10ff10+ |r");	end
		if(gold > 0)			then	text = text..format("|cffffd700%s%s|r", goldtext, GOLD_AMOUNT_SYMBOL);		force = true;	end
		if(silver > 0 or force)	then	text = text..format(' |cffc7c7cf%d%s|r', silver, SILVER_AMOUNT_SYMBOL);	force = true;	end
		if(copper > 0 or force)	then	text = text..format(' |cffeda55f%d%s|r', copper, COPPER_AMOUNT_SYMBOL);	end
	end
	return text
end


--------------------------------------------------------

function BBar_SetDefaults()

	for i = 1, g_BBar.TOTALITEMS, 1 do		g_BBar.BBarObjects_Shown[i] = 0;	end
	g_BBar.BBarObjects_Shown[2] = 1;	-- Show Gold
	g_BBar.BBarObjectsItems = {};
	g_BBar.BBarObjectsItems_Shown = {};
	g_BBar.BBarObjectsReps_Faction = {};
	g_BBar.BBarObjectsProfs_Shown = {0, 0, 0, 0, 0, 0};

	g_BBarO.WindowLock = 0;
	g_BBarO.ShowHintTT = 1;
	g_BBarO.ShowInfoTT = 1;
	g_BBarO.MinOnCombat = 0;
	g_BBarO.MinOnInst = 0;
	g_BBarO.SliderValues = {18, 1, 18, 1};
	g_BBarO.SliderOldValues = {0, 0, 0, 0};

	-- Skip gold
end


-- Convert between arrays and strings for saving
function ValuesToString()	-- Saving

	-- Store the current gold as a string of numbers
	BBarMoney = "";
	BBarMoney = BBarMoney..tostring(g_BBar.CurrentDay).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldToday[1]).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldToday[2]).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldYesterday[1]).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldYesterday[2]).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldThisWeek[1]).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldThisWeek[2]).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldLastWeek[1]).." ";
	BBarMoney = BBarMoney..tostring(g_BBar.GoldLastWeek[2]).." ";

	-- Store the currencies shown state as a string of 1/0s
	BBarShownCurrency = "";
	for i = 1, g_BBar.TOTALITEMS, 1 do
		BBarShownCurrency = BBarShownCurrency..tostring(g_BBar.BBarObjects_Shown[i]);
	end

	-- Store the custom items as strings of numbers and 1/0s
	BBarCustomItems = "";
	BBarShownItems = "";
	for i = 1, getn(g_BBar.BBarObjectsItems), 1 do
		if(g_BBar.BBarObjectsItems[i] > 0) then
			BBarCustomItems = BBarCustomItems..tostring(g_BBar.BBarObjectsItems[i]).." ";
			BBarShownItems = BBarShownItems..tostring(g_BBar.BBarObjectsItems_Shown[i]);
		end
	end

	-- Store the reputations as <position> <faction>
	BBarReputations = "";
	for pos, values in pairs(g_BBar.BBarObjectsReps_Faction) do
		if(values[2] > 0) then		-- Only save the ones shown
			BBarReputations = BBarReputations..tostring(pos).." ";
			BBarReputations = BBarReputations..tostring(values[1]).." ";
		end
	end

	-- Store the professions
	BBarProfessions = "";
	for i = 1, g_BBar.MAXPROFESSIONS, 1 do
		BBarProfessions = BBarProfessions..tostring(g_BBar.BBarObjectsProfs_Shown[i]);
	end
	
	BBar_SaveLoadSettingsOptions(1);
end

function StringToValues()	-- Loading

	BBar_SetDefaults();

	local moneys = { strsplit(" ", BBarMoney) };
	g_BBar.CurrentDay = tonumber(moneys[1]);
	g_BBar.GoldToday[1] = tonumber(moneys[2]);
	g_BBar.GoldToday[2] = tonumber(moneys[3]);
	g_BBar.GoldYesterday[1] = tonumber(moneys[4]);
	g_BBar.GoldYesterday[2] = tonumber(moneys[5]);
	g_BBar.GoldThisWeek[1] = tonumber(moneys[6]);
	g_BBar.GoldThisWeek[2] = tonumber(moneys[7]);
	g_BBar.GoldLastWeek[1] = tonumber(moneys[8]);
	g_BBar.GoldLastWeek[2] = tonumber(moneys[9]);

	for i = 1, g_BBar.TOTALITEMS, 1 do
		if(strbyte(BBarShownCurrency, i) == nil) then	break;	end
		g_BBar.BBarObjects_Shown[i] = tonumber(strbyte(BBarShownCurrency, i)) - 48;	-- Convert ascii code to number
	end

	local items = { strsplit(" ", BBarCustomItems) };
	for i = 1, getn(items) - 1, 1 do	-- -1 as last item is always invalid
		if(tonumber(items[i]) > 0) then
			g_BBar.BBarObjectsItems[i] = tonumber(items[i]);
			g_BBar.BBarObjectsItems_Shown[i] = tonumber(strbyte(BBarShownItems, i)) - 48;	-- Convert ascii code to number
		end
	end
	
	local count = 0;
	local pos = 0;
	items = { strsplit(" ", BBarReputations) };
	for i = 1, getn(items) - 1, 1 do	-- -1 as last item is always invalid
		if(tonumber(items[i]) > 0) then
			if(count == 0) then	pos = tonumber(items[i]);
			else	g_BBar.BBarObjectsReps_Faction[pos] = {tonumber(items[i]), 1};
			end
			count = (1 - count);
		end
	end
	
	for i = 1, g_BBar.MAXPROFESSIONS, 1 do
		if(strbyte(BBarProfessions, i) == nil) then	break;	end
		g_BBar.BBarObjectsProfs_Shown[i] = tonumber(strbyte(BBarProfessions, i)) - 48;	-- Convert ascii code to number
	end

	BBar_SaveLoadSettingsOptions(0);
end


function _T(text)			DEFAULT_CHAT_FRAME:AddMessage(text, 1, 1, 1);	end
function _T(text, r, g, b)	DEFAULT_CHAT_FRAME:AddMessage(text, r, g, b);	end
function _C(cond, a, b)		if cond then return a else return b end			end


---------------------------------------------------------------------------
---------------------------------------------------------------------------
--- Unused


--[[
local BBarObjects_Points = {395, 396, 392, 390};
local BBarObjects_Professions = {61, 361, 698, 402, 81};
local BBarObjects_Items = {515, 241, 416, 391, 614, 615, 697};
local BBarObjects_Archaeology = {384, 398, 394, 385, 397, 393, 400, 401, 676, 677};

390 = Conquest Points
392 = Honor Points
395 = Justice Points
396 = Valor Points

61 = Dalaran Jewelcrafter's Token
361 = Illustrious Jewelcrafter's Token
698 = Zen Jewelcrafter's Token
402 = Ironpaw Token
81 = Epicurean's Award

515 = Darkmoon Prize Ticket
241 = Champion's Seal
416 = Mark of the World Tree
391 = Tol Barad Commendation
614 = Mote of Darkness
615 = Essence of Corrupted Deathwing
697 = Elder Charm of Good Fortune

384 = Dwarf Archaeology Fragment
398 = Draenei Archaeology Fragment
394 = Night Elf Archaeology Fragment
385 = Troll Archaeology Fragment
397 = Orc Archaeology Fragment
393 = Fossil Archaeology Fragment
399 = Vrykul Archaeology Fragment
400 = Nerubian Archaeology Fragment
401 = Tol'vir Archaeology Fragment
676 = Pandaren Archaeology Fragment
677 = Mogu Archaeology Fragment
]]--

